package Sourceyard::User::Template;
use parent 'App::Sourceyard::User::Template';
use strict;
use warnings;
use Carp;

sub new {
    my $class = shift;
    my $ctl = shift or croak 'not enough arguments';
    my $self = $class->SUPER::new;
    $self->{_controller} = $ctl;
    $self->set(@_);
    return $self;
}

sub controller {
    my ($self) = @_;
    return $self->{_controller};
}

sub user {
    my ($self) = @_;
    return $self->controller->stash('user');
}

sub _validate_username {
    my ($self, $name) = @_;
    my $res = $self->SUPER::_validate_username($name);
    if ($res) {
	if ($self->controller
	         ->db
	         ->resultset('User')
	         ->search({ user_name => $name  })->first) {
	    $self->error(username => 'Username already exists');
	    return 0;
	}
    }
    return $res;
}

sub _validate_password {
    my ($self, $pass) = @_;
    my $res = $self->SUPER::_validate_password($pass);
    if ($res && $self->user && $self->user->user_pw->check($pass)) {
	$self->error(password => "Password didn't change");
	return 0;
    }
    return $res;
}
    

1;
